/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : Ftm Lpit Lptmr Port_Ci LpCmp
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 1.0.0
*   Build Version        : S32K1_RTD_1_0_0_HF01_D2109_ASR_REL_4_4_REV_0000_20210907
*
*   (c) Copyright 2020-2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef LPIT_ICU_IP_VS_0_PBCFG_H
#define LPIT_ICU_IP_VS_0_PBCFG_H

/**
 * @file    Lpit_Icu_VS_0_PBcfg.h
 * @version 1.0.0
 *
 * @brief   AUTOSAR Icu - contains the data exported by the ICU module.
 * @details Contains the information that will be exported by the module, as requested by AUTOSAR.
 *
 * @addtogroup lpit_icu_ip LPIT IPL
 * @{
 */

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                         INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Lpit_Icu_Ip_Types.h"

/*==================================================================================================
*                                 SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define LPIT_ICU_IP_VS_0_PBCFG_VENDOR_ID                    43
#define LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_MAJOR_VERSION     4
#define LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_MINOR_VERSION     4
#define LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_REVISION_VERSION  0
#define LPIT_ICU_IP_VS_0_PBCFG_SW_MAJOR_VERSION             1
#define LPIT_ICU_IP_VS_0_PBCFG_SW_MINOR_VERSION             0
#define LPIT_ICU_IP_VS_0_PBCFG_SW_PATCH_VERSION             0

/*==================================================================================================
*                                       FILE VERSION CHECKS
==================================================================================================*/
/* Check if header file and ICU Lpit_Icu_Ip_Types.h file are of the same vendor. */
#if (LPIT_ICU_IP_VS_0_PBCFG_VENDOR_ID != LPIT_ICU_IP_TYPES_VENDOR_ID)
    #error "Lpit_Icu_Ip_VS_0_PBcfg.h and Lpit_Icu_Ip_Types.h have different vendor IDs!"
#endif
/* Check if header file and ICU Lpit_Icu_Ip_Types.h file are of the same AutoSar version */
#if ((LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_MAJOR_VERSION    != LPIT_ICU_IP_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_MINOR_VERSION    != LPIT_ICU_IP_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (LPIT_ICU_IP_VS_0_PBCFG_AR_RELEASE_REVISION_VERSION != LPIT_ICU_IP_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Lpit_Icu_Ip_VS_0_PBcfg.h and Lpit_Icu_Ip_Types.h are different!"
#endif
/* Check if header file and ICU Lpit_Icu_Ip_Types.h file are of the same Software version */
#if ((LPIT_ICU_IP_VS_0_PBCFG_SW_MAJOR_VERSION != LPIT_ICU_IP_TYPES_SW_MAJOR_VERSION) || \
     (LPIT_ICU_IP_VS_0_PBCFG_SW_MINOR_VERSION != LPIT_ICU_IP_TYPES_SW_MINOR_VERSION) || \
     (LPIT_ICU_IP_VS_0_PBCFG_SW_PATCH_VERSION != LPIT_ICU_IP_TYPES_SW_PATCH_VERSION))
#error "Software Version Numbers of Lpit_Icu_Ip_VS_0_PBcfg.h and Lpit_Icu_Ip_Types.h are different!"
#endif

/*==================================================================================================
*                                        GLOBAL CONSTANTS
==================================================================================================*/

#ifdef __cplusplus
}
#endif

/** @} */

#endif /* LPIT_ICU_IP_VS_0_PBCFG_H */
